import tkinter as tk
from controller import myController
from config import cwd
from config import myFrameHeight
from config import myFrameWidth



class GPSPage(tk.Frame):
    firstFixFlag = False # this will go true after the first GPS fix.
    firstFixDate = ""

    def ButtonBackToMain_click(self, appClass):
        print("GO TO STARTPAGE")
        appClass.show_start_page()

    # Helper function to take HHMM.SS, Hemisphere and make it decimal:
    def degrees_to_decimal(self,data, hemisphere):
        try:
            decimalPointPosition = data.index('.')
            degrees = float(data[:decimalPointPosition-2])
            minutes = float(data[decimalPointPosition-2:])/60
            output = degrees + minutes
            if hemisphere is 'N' or hemisphere is 'E':
                return output
            if hemisphere is 'S' or hemisphere is 'W':
                return -output
        except:
            return ""


    # Helper function to take a $GPRMC sentence, and turn it into a Python dictionary.
    # This also calls degrees_to_decimal and stores the decimal values as well.
    def parse_GPRMC(self,data):
        data = data.split(',')
        dict = {
                'fix_time': data[1],
                'validity': data[2],
                'latitude': data[3],
                'latitude_hemisphere' : data[4],
                'longitude' : data[5],
                'longitude_hemisphere' : data[6],
                'speed': data[7],
                'true_course': data[8],
                'fix_date': data[9],
                'variation': data[10],
                'variation_e_w' : data[11],
                'checksum' : data[12]
        }
        dict['decimal_latitude'] = self.degrees_to_decimal(dict['latitude'], dict['latitude_hemisphere'])
        dict['decimal_longitude'] = self.degrees_to_decimal(dict['longitude'], dict['longitude_hemisphere'])
        return dict


    def BtnGpsRecordStart_click(self, appClass):
        self.BtnGpsRecordStart.configure(state='disabled')
        self.BtnGpsRecordStop.configure(state='normal')
        myController.sendControlCenterMode('4','0','65000') #gps due factory rate for 65000 data

    def BtnGpsRecordStop_click(self, appClass):
        self.BtnGpsRecordStart.configure(state='normal')
        self.BtnGpsRecordStop.configure(state='disabled')
        myController.sendControlCenterMode('4','0','0') #gps due factory rate for 0 data so stop


    def __init__(self, parent, appClass):

        tk.Frame.__init__(self, parent, height=myFrameHeight, width=myFrameWidth)
        self.imgBack=tk.PhotoImage(file=cwd + "/icons/back1.png")
        self.ButtonBack1 = tk.Button(self, image=self.imgBack, command = lambda: self.ButtonBackToMain_click(appClass))
        self.ButtonBack1.place(x=myFrameWidth -120, y=myFrameHeight -120, height=120, width=120)

        self.txtGpsRecu = tk.Text(self)
        self.ScrolltxtGpsRecu = tk.Scrollbar(self.txtGpsRecu)
        self.ScrolltxtGpsRecu.pack(side=tk.RIGHT, fill=tk.Y)
        self.txtGpsRecu.pack(side=tk.LEFT, fill=tk.Y)
        self.ScrolltxtGpsRecu.config(command=self.txtGpsRecu.yview)
        self.txtGpsRecu.config(yscrollcommand=self.ScrolltxtGpsRecu.set)
        self.txtGpsRecu.place(x=5,y=35,anchor='nw',width=600, height=300)
        self.BtnGpsRecordStart= tk.Button(self,command=lambda: self.BtnGpsRecordStart_click(appClass),text="Start")
        self.BtnGpsRecordStart.place(x=5,y=5, height=25, width=60)
        self.BtnGpsRecordStop= tk.Button(self,command=lambda: self.BtnGpsRecordStop_click(appClass),text="Stop")
        self.BtnGpsRecordStop.place(x=150,y=5, height=25, width=60)

      

    def OnReceiveSerialData(self, dataObj):
        line = str(dataObj.GpsRmcMessage)
        #print (str(line))
        
                     
        dataObj.GpsRmcMessage=""

        if "$GNRMC" in line: # This will exclude other NMEA sentences the GPS unit provides.
            self.txtGpsRecu.insert('1.0', line +'\n')

            gpsData = self.parse_GPRMC(str(line)) # Turn a GPRMC sentence into a Python dictionary called gpsData
            if gpsData['validity'] == "A": # If the sentence shows that there's a fix, then we can log the line
                if self.firstFixFlag is False: # If we haven't found a fix before, then set the filename prefix with GPS date & time.
                    self.firstFixDate = gpsData['fix_date'] + "-" + gpsData['fix_time']
                    self.firstFixFlag = True
                
                else: # write the data to a simple log file and then the raw data as well:
                    with open(cwd+"/gpsdata/" + self.firstFixDate +".txt", "a") as myfile:
                        myfile.write(gpsData['fix_date'] + "," + gpsData['fix_time'] + "," + str(gpsData['decimal_latitude']) + "," + str(gpsData['decimal_longitude']) +"\n")
                        #with open("/home/pi/gps_experimentation/" + self.firstFixDate +"-gprmc-raw-log.txt", "a") as myfile:
                        #myfile.write(line +"\n")




       
